function [ u, stat ] = mindInversion1dADMM(v, K, Kh, k, intv, weights, scalePenalties, ui, verbose, lambda, maxit)
%MINDINVERSION1DADMM Minimize homogeneous Soblev norm under the
%multiresolution constraint for SIP with d = 1, i.e. 
%
%   min 1/2 ||D^k u||_2^2
%   s.t. (w_B)^(-1) | < e_B, K u - v > | - q_B <= 0
%
%Usage:
% [u,stat] = mindInversion1dADMM(v,K,Kh,k,cube,weights,scalePenalties, ...
%                                                 ui,verbose,lambda,maxit);
% u = mindInversion1dADMM(v,K,Kh,k,cube,[],scalePenalties);
%
%Input:
% v:               data
% K:               forward operator (function handle or matrix)
% Kh:              adjoint operator (function handle or matrix)
% k:               Soblev index
% intv:            system of intervals, a list of "li" and "ri"
% weights:         weights (i.e. w_B) default: sqrt of interval lengths
% scalePenalties:  scale penalties (i.e. q_B) 
% ui:              initial value (optional)
% verbose:         level of screen print (optional)
% lambda:          coefficient of augmented Lagrangian (optional)
% maxit:           maximal number of iterations (optional)
%
%Output:
% u:      solution
% stat:   record results of each outer iteration
%
%Algorithm: Alternating Direction Method of Multipliers (ADMM)

% Housen Li @ MPIbpc
% 03.01.2015 create the code

% default input
if ~isa(K, 'function_handle')
    K = @(par) K * par;
end
if ~isa(Kh, 'function_handle')
    Kh = @(par) Kh * par;
end
if isempty(weights)
    weights = sqrt(intv.ri-intv.li+1);
else
    weights = weights(:); % change to column vector
end
if isscalar(scalePenalties)
    scalePenalties = ones(size(weights)).*scalePenalties;
else
    scalePenalties = scalePenalties(:);
end
if (nargin < 8) || isempty(ui)
    ui = Kh(0.*v);
end
if (nargin < 9) || isempty(verbose)
    verbose = 0;
end
if (nargin < 10) || isempty(lambda)
    lambda   = 0.02;
end
if (nargin < 11) || isempty(maxit)
    maxit    = 1e3;
end

% default parameters
dif_tol  = 1e-5;
gap_tol  = 1e-5;

maxit_dyk          = 500; 
increment_tol_dyk  = 1e-5;
constraint_tol_dyk = 1e-3;

maxit_sob = 500;
tol_sob   = 1e-5;

% initialization
cit   = 0;
uo    = ui; 
w     = 0.*v;
p     = 0.*v;
Kuo   = K(uo);

normv = norm(v, 'fro'); 
gap   = norm(Kuo - v + w, 'fro'); 
done  = 0;
check = 50;

if nargout > 1
    stat = cell(maxit, 1);
end
if verbose > 0
    fprintf('MR-Sobolev inversion via ADMM ...\n');
end

% ADMM iteration
while ~done
    cit = cit + 1;
    w   = projectionMLSball1d(p+v-Kuo, intv, weights, scalePenalties, ...
        maxit_dyk, increment_tol_dyk, constraint_tol_dyk, verbose-1);
    u   = sobolevInversion1d(p+v-w, K, Kh, lambda, k, uo, maxit_sob, tol_sob, verbose-1);
    u   = u - mean(u); % mean zero
    Ku  = K(u);
    p   = p - (Ku + w - v);
    
    gap = norm(Ku + w - v, 'fro');
    dif = norm(Ku - Kuo, 'fro');
    if (cit >= maxit) || (dif <= dif_tol*normv && min(gap,gap/normv) <= gap_tol)
        done = 1;
    end
    if (mod(cit,check) == 0) && (verbose > 1)  
        fprintf('ADMM iteration %d\n', cit);
        fprintf('          gap to equality: %.5f\n', gap);
        fprintf('          size of increment (in image space): %.5f\n', dif);
        if verbose > 2
            fprintf('          size of increment (in preimage space): %.5f\n', norm(u-uo,'fro'));
        end
    end
    if verbose > 3
        imagesc(u);
        title(sprintf('iteration %d', cit));
        drawnow;
    end
    uo  = u;
    Kuo = Ku;
    if nargout > 1
        stat{cit} = u;
    end
end

if nargout > 1
    stat = stat(1:cit);
end
if verbose > 0
    fprintf(' stop at: # iterations: %d / %d\n', cit, maxit);
    if verbose > 1
        fprintf('          gap to equality: %.5f\n', gap);
        fprintf('          size of increment: %.5f\n', dif);
    end
end

% Equivalent optimization problem
% 
%   minimize ||D^k u||_{2}^2 + Chi(w)
%   subject to K u + w - v = 0
%
% Here Chi(w) = { 0     if (w_B)^(-1) | < e_B, w > | - q_B <= 0
%               { inf   otherwise
%
% Algorithm: Alternating Direction Method of Multipliers (ADMM)
%   augmented Lagrangian
%        L_{lambda}(u, w, p) = ||D^k u||_{2}^2 + Chi(w) ...
%           - 1/lambda <p, K u + w - v> + 1/(2 lambda)||K u + w - v||_{2}^2
%       
%   iterations
%       w := argmin_{w} L_{lambda}(u,w,p)
%       u := argmin_{u} L_{lambda}(u,w,p)
%       p := p - (K u + w - v)