function [ u, stat ] = mindInversion1dQP( v, Km, k, interval, weights, scalePenalties, algorithm )
%MINDINVERSION1DQP MIND for inversion of 'Km u = v' via built-in function
%quadprog
%
%          min || D^k u ||_2
%   subject to (w_I)^(-1) | < e_I, Km u - v > | - q_I <= 0
%
%Usage:
%   [u, stat] = mindInversion1dQP(v, Km, k, interval, weights, ...
%                                                scalePenalties, algorithm)
%   u = mindInversion1dQP(v, Km, k, interval, [], scalePenalties)
%
%Input:
%   v:              a vector (m x 1), noisy blurred data
%   Km:             a matrix (m x n), forward operator
%   k:              a scalar, Sobolev index
%   interval:       a list (li, ri), system of intervals
%   weights:        a vector, weights for each interval (i.e. w_I)
%   scalePenalties: a vector, scale penalties for each interval (i.e. q_I)
%   algorithm:      algorithm for 'quadprog' (interior point method by 
%                   default)
%
%Output:
%   u:        the solution         
%   stat:     statistics for iterations
%       fval:     value of objective function
%       exitflag: integer identifying the reason the algorithm terminated
%       output:   structure containing information about the optimization
%       lambda:   structure containing the Lagrange multipliers at the 
%                 solution
%   see 'quadprog' for further details about the last four return values

% Implementation detail
% (a) H^k-norm is discretized via finite differences

% Housen Li @ MPIbpc
% 16.12.2015 build the code: this is a correction for 
%            'mindDeconvolution1dQP.m' by removing the requirement that the
%            solution should be a spline
% 05.01.2016 add variable 'algorithm'



% Default input
ni = length(interval.li);
if isempty(weights)
    weights = sqrt(interval.ri - interval.li + 1);
end
weights = weights(:);
if isscalar(scalePenalties)
    scalePenalties = ones(ni,1).*scalePenalties;
end
scalePenalties = scalePenalties(:);
if nargin < 7
    algorithm = 'interior-point-convex';
end

m  = length(v);
v  = v(:);
ui = Km' * v;
n  = length(ui);

% Optimization parameters
tolfun    = 1e-10;
tolcon    = 1e-10;
tolx      = 1e-10;
maxit     = 200;

% Build optimization problem
%       Finite difference
D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
D(1,n) = -1;
Dk     = D^k;

H = n^(2*k-1) * (Dk' * Dk);

%       Multiscale constraint
B = zeros(2*ni, m);
for i = 1:ni
    len = 1 / weights(i);
    B(i,   interval.li(i):interval.ri(i)) = len;
    B(i+ni,interval.li(i):interval.ri(i)) = -len;
end
B = sparse(B);
b = B * v + [scalePenalties; scalePenalties];
B = B * Km;
% opts = optimoptions('quadprog', 'Algorithm', algorithm, 'Display', 'off');
opts = optimoptions('quadprog', 'Algorithm', algorithm, 'Display', 'iter-detailed', 'Diagnostics', 'off', ...
    'TolFun', tolfun, 'TolCon', tolcon, 'TolX', tolx, 'MaxIter', maxit);

%       Mean zero
Aeq = ones(1,n);
beq = 0;

% Initial value
x0 = ui;

% Quadratic programming
fprintf('Start quadratic programming ...\n');
% [u, fval, exitflag, output, lambda] = quadprog(H, zeros(n,1), B, b, [], [], [], [], x0, opts);
[u, fval, exitflag, output, lambda] = quadprog(H, zeros(n,1), B, b, Aeq, beq, [], [], x0, opts);

stat.fval     = fval;
stat.exitflag = exitflag;
stat.output   = output;
stat.lambda   = lambda;

end

