function [ u, stat ] = mindRegression1dADMM(v, k, intv, weights, scalePenalties, ui, verbose, lambda, maxit)
%MINDREGRESSION1DADMM Minimize homogeneous Soblev norm under the
%multiresolution constraint for one-dimensional regression, i.e. 
%
%   min 1/2 ||D^k u||_2^2
%   s.t. (w_B)^(-1) | < e_B, u - v > | - q_B <= 0
%
%Usage:
% [u,stat] = mindRegression1dADMM(v,k,intv,weights,scalePenalties,...
%                                                 ui,verbose,lambda,maxit);
% u = mindRegression1dADMM(v,k,intv,[],scalePenalties);
%
%Input:
% v:               data
% k:               Soblev index
% intv:            system of intervals, a list of "li" and "ri"
% weights:         weights (i.e. w_B) default: sqrt of interval lengths 
% scalePenalties:  scale penalties (i.e. q_B) 
% ui:              initial value (optional)
% verbose:         level of screen print (optional)
% lambda:          coefficient of augmented Lagrangian (optional)
% maxit:           maximal number of iterations (optional)
%
%Output:
% u:      solution (on [0,1) regular grid)
% stat:   record results of each outer iteration
%
%Algorithm: Alternating Direction Method of Multipliers (ADMM)
%Discretization of ||D^k u||_2 by splines

% Housen Li @ MPIbpc
% 06.01.2015 create the code
% 10.12.2015 extended to list of intervals
% 05.01.2016 change name "mrConstraintSobolevReg1dADMMsp" --> 
%            "mindRegression1dADMM"


% default input
sz = size(v);
v  = v(:);
if isempty(weights)
    weights = sqrt(intv.ri-intv.li+1);
else
    weights = weights(:); % change to column vector
end
if isscalar(scalePenalties)
    scalePenalties = ones(size(weights)).*scalePenalties;
else
    scalePenalties = scalePenalties(:);
end
if (nargin < 6) || isempty(ui)
    ui = v; % 0.*v
end
if (nargin < 7) || isempty(verbose)
    verbose = 0;
end
if (nargin < 8) || isempty(lambda)
    lambda   = 0.02; % numel(v)/(2*pi*pi*numel(v))^k;
end
if (nargin < 9) || isempty(maxit)
    maxit    = 3e3; % k*500
end

% check validaty
if (length(v) < max(intv.ri)) || any(intv.ri - intv.li < 0) || ...
        (length(intv.li) ~= length(intv.ri)) || (length(intv.li) ~= length(weights)) ...
        || (length(weights) ~= length(scalePenalties))
    error('Data is not valid! Please check it!');
end    

% default parameters
dif_tol = 1e-8;
gap_tol = 1e-8;

fv_tol  = 1e-10;
con_tol = 1e-10;


maxit_dyk          = 500; 
increment_tol_dyk  = 1e-5;
constraint_tol_dyk = 1e-3;

% initialization
cit   = 0;
uo    = ui; 
w     = 0.*v;
p     = 0.*v;

normv = norm(v, 'fro'); 
done  = 0;
check = 50;
fvalo = 0;

if nargout > 1
    stat.solution     = cell(maxit, 1);
%     stat.primResidual = zeros(maxit, 1);
%     stat.dualResidual = zeros(maxit, 1);
end
if verbose > 0
    fprintf('Sobolev-Multiscale-Constrained estimator via ADMM (spline version)\n');
    fprintf('                                                                             Dykstra-Residual\n');
    fprintf('  Iter   Primal-Residual   Dual-Residual   Objective-Value   Constraint      before     after\n');
    timeADMM = tic;
end

% ADMM iteration
while cit <= maxit
    cit = cit + 1;
%     w   = dykstra_mex(lambda*p+v-uo, scl, qs, maxit_dyk, dif_tol_dyk, mr_tol_dyk, verbose-1);
    w   = projectionMLSball1d(p+v-uo, intv, weights, scalePenalties, ...
        maxit_dyk, increment_tol_dyk, constraint_tol_dyk, verbose-1);
    if verbose > 0
        dykRes0 = mrLinearStat1d(p+v-uo, intv, weights, scalePenalties);
        dykRes  = mrLinearStat1d(w, intv, weights, scalePenalties);
%         dykRes0 = mr_constraint_residual(0, p+v-uo, scl, 1, qe);
%         dykRes  = mr_constraint_residual(0, w, scl, 1, qe);
    end
    
    u   = sobolevRegression1d(p+v-w, lambda, k);
    u   = u - mean(u); % mean zero
    p   = p - (u + w - v);
    
    % stopping criteria
    gap    = norm(u + w - v, 'fro');
    dif    = norm(u - uo, 'fro');
    fval   = l2norm_der_sp(u, k, 2*k);
    conRes = mrLinearStat1d(u-v, intv, weights, scalePenalties);
%     conRes = mr_constraint_residual(u, v, scl, 1, qe);
    if min(dif, dif/normv) <= dif_tol && ...
            min(gap, gap/normv) <= gap_tol && ...
            abs(fvalo - fval) < max(fv_tol*fval, fv_tol) && ...
            conRes < con_tol
        done = 1;
        break;
    end
    if (mod(cit,20*check) == 0) && (verbose > 0)  
        fprintf('                                                                             Dykstra-Residual\n');
        fprintf('  Iter   Primal-Residual   Dual-Residual   Objective-Value   Constraint      before     after\n');
    end
    if (mod(cit,check) == 0) && (verbose > 0)  
        fprintf('%6d   %15.8g   %13.8g   %15.8g   %10.8g   %9.5g  %9.5g\n', ...
            cit, gap, dif, fval, conRes, dykRes0, dykRes);
        if (mod(cit, check*20) == 0) && (~done) && (cit > maxit)
            fprintf('                                                                             Dykstra-Residual\n');
            fprintf('\n  Iter   Primal-Residual   Dual-Residual   Objective-Value   Constraint      before     after\n');
        end
    end

    % update
    uo    = u;
    fvalo = fval;

    if nargout > 1
        stat.solution{cit}     = u;
%         stat.primResidual(cit) = gap;
%         stat.dualResidual(cit) = dif;
    end
end

if nargout > 1
    stat.solution = stat.solution(1:cit);
%     stat.primResidual = stat.primResidual(1:cit);
%     stat.dualResidual = stat.dualResidual(1:cit);
end
if verbose > 0
    if done
        fprintf('ADMM converges at %5d (max %5d) ... \n', cit, maxit);
        fprintf('                                Tol\n');
        fprintf('Primal-Residual %10.8g     %10.8g\n', gap, gap_tol);
        fprintf('Dual-Residual   %10.8g     %10.8g\n', dif, dif_tol);
        fprintf('Objective-Value %10.8g Inc:%10.8g\n', fval, fv_tol);
        fprintf('Constraint      %10.8g     %10.8g\n', conRes, con_tol);
        fprintf('elapses         %10.8g seconds\n', toc(timeADMM));
    else
        fprintf('ADMM does NOT converge: maxIt %5d is reached ... \n', maxit);
        fprintf('                                Tol\n');
        fprintf('Primal-Residual %10.8g     %10.8g\n', gap, gap_tol);
        fprintf('Dual-Residual   %10.8g     %10.8g\n', dif, dif_tol);
        fprintf('Objective-Value %10.8g Inc:%10.8g\n', fval, fv_tol);
        fprintf('Constraint      %10.8g     %10.8g\n', conRes, con_tol);
        fprintf('elapses         %10.8g seconds\n', toc(timeADMM));
    end
end

u = reshape(u, sz);

end

