function [ u, stat ] = mindRegression1dQP( v, k, intval, q, algorithm, lthDer )
%MINDREGRESSION1DQP MIND for one-dimensional regression, i.e.
%
%       min 1/2 ||D^k u||_2^2   subject to || u -v ||_{B} <= q
%
%Usage:
% [u, stat] = mindRegression1dQP(v, k, intval, q, algorithm, lthDer)
% u = mindRegression1dQP(v, k, intval, q)
%
%Input:
% v:         data
% k:         Sobolev index
% intval:    system of intervals, a list of 'li' and 'ri'
% q:         threshold
% algorithm: algorithm for 'quadprog' (interior point method by default)
% lthDer:    order of derivative estimation (0 by default)
%
%Output:
%   u:        the solution         
%   stat:     statistics for iterations
%       fval:     value of objective function
%       exitflag: integer identifying the reason the algorithm terminated
%       output:   structure containing information about the optimization
%       lambda:   structure containing the Lagrange multipliers at the 
%                 solution
%   see 'quadprog' for further details about the last four return values

% Housen Li @ MPIbpc
% 13.12.2015 add estimation of derivatives
% 05.01.2016 change name "mrConstraintSobolevReg1dQPspcoef" -->
%            "mindRegression1dQP"
%            change variables "v, k, intval, q, sigma, algorithm, lthDer" 
%            --> "(v, k, intval, q, algorithm, lthDer)"

if nargin < 6
    algorithm = 'interior-point-convex';
end
if nargin < 7
    lthDer = 0;
end

% default parameters
tolfun = 1e-10;
tolcon = 1e-10;
tolx   = 1e-10;
maxit  = 200;

sz = size(v);
v  = v(:);
n  = numel(v);
ni = length(intval.li);

% B-spline bases matrix
x = linspace(0, 1, n+1);
x = x(1:end-1);
a = nbspline(x(:), 2*k, 0, n);
Q = sparse(toeplitz(a, a([1, end:-1:2])));

% Gramian matrix
a = zeros(1, n);
f0 = @(par) nbspline(par, k, 0, n);
for i = -(k-1):(k-1)
    fi = @(par) nbspline(par, k, i, n);
    ip = @(par) fi(par) .* f0(par);
    a(mod(i, n)+1) = quadgk(ip, 0, 1);
end
G = sparse(toeplitz(a));
H = blkdiag(sparse(n, n), G);
H = 0.5*(H+H');

D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
D(1,n) = -1;
if lthDer > 0, Dl = D^l; end
D      = sqrt(n)^k * D^k;
% D      = n^k * D^k;

% inequality matrix
B = zeros(2*ni, n);
for i = 1:ni
    len = 1/sqrt(intval.ri(i) - intval.li(i) + 1);
    B(i,   intval.li(i):intval.ri(i)) = len;
    B(i+ni,intval.li(i):intval.ri(i)) = -len;
end
B = sparse(B);
b = q + B*v;
B = B * Q;
B = sparse([B, sparse(2*ni,n)]);
Aeq = sparse([ones(1,2*n); D, -speye(n,n)]);
beq = zeros(n+1,1);

% opts = optimset('Algorithm', algorithm, 'Display', 'off');
% opts = optimoptions('quadprog', 'Algorithm', algorithm, 'Display', 'off');
opts = optimoptions('quadprog', 'Algorithm', algorithm, 'Display', 'iter-detailed', 'Diagnostics', 'off', ...
    'TolFun', tolfun, 'TolCon', tolcon, 'TolX', tolx, 'MaxIter', maxit);

% % total relative error
% fprintf('|| H || = %g\n', normest(H));
% fprintf('|| B || = %g\n', normest(B(:,1:n)));
% fprintf('|| b || = %g\n', norm(b));
% fprintf('|| Aeq || = %g\n', normest(Aeq));

% initial value
x0  = zeros(2*n, 1);

% quadratic programming
fprintf('Start quadratic programming ...\n');
[c, fval, exitflag, output, lambda] = ...
    quadprog(H, zeros(2*n,1), B, b, Aeq, beq, [], [], x0, opts);

if lthDer > 0 
    u = Q * (Dl * c(1:n));
else
    u = Q * c(1:n);
end
u = reshape(u, sz);

stat.fval     = fval;
stat.exitflag = exitflag;
stat.output   = output;
stat.lambda   = lambda;


end

