function [ u, stat ] = mindtvInversion2dADMM( v, K, Kh, cube, weights, scalePenalties, ui, opts )
%MINDTVINVERSION2DADMM Minimize total variation (TV) under the 
%multiresolution constraint, i.e. 
%
%   min  ||\nabla u||_1
%   s.t. (w_B)^(-1) | < e_B, K u - v > | - q_B <= 0
%
%Usage:
% [u,stat] = mindtvInversion2dADMM(v,K,Kh,cube,weights,scalePenalties,...
%                                  ui,verbose,lambda,maxit);
% u = mindtvInversion2dADMM(v,K,Kh,cube,[],scalePenalties);
%
%Input:
% v:                     data
% K:                     forward operator (function handle or matrix)
% Kh:                    adjoint operator (function handle or matrix)
% cube:                  system of cubes
% weights:               weights (i.e. w_B)
% scalePenalties:        scale penalties (i.e. q_B) 
% ui:                    initial value (optional)
% opts                   (optional)
%       verbose:         level of screen print
%       lambda:          coefficient of augmented Lagrangian
%       maxit:           maximal number of iterations
%       it_param_tvinv:  iteration parameters for TV inversion
%       vis_param_tvinv: visualization parameters for TV inversion
%       ...              see function parseInput for more details
%
%Output:
% u:       solution
% stat:    details of each outer iteration         
%   objectVal:  objective values ||\nabla u||_1
%   constrGap:  gaps of the constraint (w_B)^(-1) | < e_B, u - v > | - q_B 
%
%Algorithm: Alternating Direction Method of Multipliers (ADMM)

% Housen Li @ MPIbpc
% 30.10.2014 create the code
% 25.09.2015 make sure the code works properly
% 06.10.2015 extend to general weights and scale penalties
% 12.10.2015 add visulization
% 27.10.2015 add opts parameter which consists of all tuning parameters
% 07.01.2016 change name "mrConstraintTVinversion2dADMM" --> 
%            "mindtvInversion2dADMM"

% default input
if ~isa(K, 'function_handle')
    K = @(par) K * par;
end
if ~isa(Kh, 'function_handle')
    Kh = @(par) Kh * par;
end
if isempty(weights)
    weights = sqrt(prod(cube.ed-cube.st+1,2));
end
if isscalar(scalePenalties)
    scalePenalties = ones(length(cube.st),1).*scalePenalties;
end
if nargin < 7
    ui = Kh(0.*v);
end
if nargin < 8
    opts = [];
end
[verbose, check, lambda, maxit, dif_tol, gap_tol, ...
    maxit_dyk, increment_tol_dyk, constraint_tol_dyk, ...
    it_param_tvinv, vis_param_tvinv] = parseInput(opts);

% initialization
cit   = 0;
uo    = ui; 
w     = 0.*v;
p     = 0.*v;
Kuo   = K(uo);

normv = norm(v, 'fro'); 
gap   = norm(Kuo - v + w, 'fro'); 
done  = 0;

if nargout > 1
    stat.objectVal = zeros(maxit, 1);
    stat.constrGap = zeros(maxit, 1);
end
if verbose > 0
    fprintf('MIND-TV via ADMM ...\n');
    timeADMM = tic;
end

% ADMM iteration
while ~done
    cit = cit + 1;
    w = projectionMLSball2d(p+v-Kuo, cube, weights, scalePenalties, maxit_dyk, increment_tol_dyk, constraint_tol_dyk, verbose-1);
    if verbose > 1
        fprintf('TV inversion ...\n');
        timeTV = tic;
    end
    u = tvInversion2dADMM(p+v-w, K, Kh, lambda, it_param_tvinv, vis_param_tvinv);
    if verbose > 1
        fprintf('elapse %g\n', toc(timeTV));
    end
    Ku = K(u);
    p  = p - (Ku + w - v);
    
    gap = norm(Ku + w - v, 'fro');
    dif = norm(Ku - Kuo, 'fro');
    if (cit >= maxit) || (dif <= dif_tol*normv && min(gap,gap/normv) <= gap_tol)
        done = 1;
    end
    if mod(cit,check) == 0  
        fprintf('ADMM iteration %d\n', cit);
        if verbose > 1
            fprintf('          gap to equality: %.5f\n', gap);
            fprintf('          size of increment (in image space): %.5f\n', dif);
            if verbose > 2
                fprintf('          size of increment (in preimage space): %.5f\n', norm(u-uo,'fro'));
            end
        end
    end
    if verbose > 3
        figure(1); clf;
        subplot(221); imagesc(v); title('data'); axis off; colorbar; drawnow;
        subplot(222); imagesc(u); title(sprintf('iteration %d', cit)); axis off; colorbar; drawnow;
        subplot(223); imagesc(v - Ku); title('residual'); axis off; colorbar; drawnow;
        subplot(224); imagesc(uo); title('previous iteration'); axis off; colorbar; drawnow;
    end
    uo  = u;
    Kuo = Ku;
    if nargout > 1
        [u_x, u_y] = num_grad(u);
        objectVal  = sum(sum(sqrt(u_x.^2 + u_y.^2)));
        stat.objectVal(cit) = objectVal;
        constrGap = mrLinearStat2d(Ku - v, cube, weights, scalePenalties);
        stat.constrGap(cit) = constrGap;
        if mod(cit,check) == 0
            fprintf('          objective value = %g\n', objectVal);
            fprintf('          constraint gap = %g\n', constrGap);
        end
    end
end

if nargout > 1
    stat.objectVal = stat.objectVal(1:cit);
    stat.constrGap = stat.constrGap(1:cit);
end
if verbose > 0
    fprintf('elapse %g s\n', toc(timeADMM));
    fprintf('stop at: # iterations: %d / %d\n', cit, maxit);
    if verbose > 1
        fprintf('          gap to equality: %.5f\n', gap);
        fprintf('          size of increment: %.5f\n', dif);
    end
end

% Equivalent optimization problem
% 
%   minimize ||\nabla u||_1 + Chi(w)
%   subject to K u + w - v = 0
%
% Here Chi(w) = { 0     if (w_B)^(-1) | < e_B, w > | - q_B <= 0
%               { inf   otherwise
%
% Algorithm: Alternating Direction Method of Multipliers (ADMM)
%   augmented Lagrangian
%        L_{lambda}(u, w, p) = ||\nabla u||_1 + Chi(w) ...
%           - 1/lambda <p, K u + w - v> + 1/(2 lambda)||K u + w - v||_{2}^2
%       
%   iterations
%       w := argmin_{w} L_{lambda}(u,w,p)
%       u := argmin_{u} L_{lambda}(u,w,p)
%       p := p - (K u + w - v)

%% parsing input
function [verbose, check, lambda, maxit, dif_tol, gap_tol, ...
    maxit_dyk, increment_tol_dyk, constraint_tol_dyk, ...
    it_param_tvinv, vis_param_tvinv] = parseInput(opts)

% default values
%   main iteration
verbose = 0;
check   = 50;
lambda  = 0.1;
maxit   = 1e3;
dif_tol = 1e-5;
gap_tol = 1e-5;
%   Dykstra projection
maxit_dyk          = 1e3; 
increment_tol_dyk  = 1e-5;
constraint_tol_dyk = 1e-3;
%   TV inversion
if isfield(opts, 'it_param_tvinv')
    it_param_tvinv = opts.it_param_tvinv;
else
    it_param_tvinv = [];
end
if isfield(opts, 'vis_param_tvinv')
    vis_param_tvinv = opts.vis_param_tvinv;
else
    vis_param_tvinv = [];
end

% update values
%   main iteration
if isfield(opts, 'verbose')
    verbose = opts.verbose;    
end
if isfield(opts, 'check')
    check = opts.check;    
end
if isfield(opts, 'lambda')
    lambda = opts.lambda;    
end
if isfield(opts, 'maxit')
    maxit = opts.maxit;
end
if isfield(opts, 'dif_tol')
    dif_tol = opts.dif_tol;
end
if isfield(opts, 'gap_tol')
    gap_tol = opts.gap_tol;
end
%   Dykstra projection
if isfield(opts, 'maxit_dyk')
    maxit_dyk = opts.maxit_dyk;
end
if isfield(opts, 'increment_tol_dyk')
    increment_tol_dyk = opts.increment_tol_dyk;
end
if isfield(opts, 'constraint_tol_dyk')
    constraint_tol_dyk = opts.constraint_tol_dyk;
end
%   TV inversion
if ~isfield(it_param_tvinv, 'sigma')
    it_param_tvinv.sigma = 5;
end
if ~isfield(vis_param_tvinv, 'verbose')
    vis_param_tvinv.verbose = verbose-1;
end
if ~isfield(vis_param_tvinv, 'check')
    vis_param_tvinv.check = 10;
end