function [ u, stat ] = mindtvRegression2dADMM(v, cube, weights, scalePenalties, ...
    ui, verbose, lambda, maxit)
%MINDTVREGRESSION2DADMM Minimize total variation (TV) under the
%multiresolution constraint for d = 2, i.e. 
%
%   min  ||\nabla u||_1
%   s.t. (w_B)^(-1) | < e_B, u - v > | - q_B <= 0
%
%Usage:
%
% [u,stat] = mindtvRegression2dADMM(v,cube,weights,scalePenalties,ui, ...
%                                                    verbose,lambda,maxit);
% u = mindtvRegression2dADMM(v,cube,[],scalePenalties);
%
%Input:
% v:               data
% cube:            system of cubes, a list of 'st' and 'ed'
% weights:         weights (i.e. w_B)
% scalePenalties:  scale penalties (i.e. q_B) 
% ui:              initial value (optional)
% verbose:         level of screen print
% lambda:          coefficient of augmented Lagrangian
% maxit:           maximal number of iterations
%
%Output:
% u:      solution
% stat:   record results of each outer iteration
%
%Algorithm: Alternating Direction Method of Multipliers (ADMM)

% Housen Li @ MPIbpc
% 30.10.2014 create the code
% 25.09.2015 make sure the code works properly
% 06.10.2015 extend to general weights and scale penalties
% 05.01.2016 change name "mrConstraintTVregression2dADMM" --> 
%            "mindtvRegression2dADMM"

% default input
if isempty(weights)
    weights = sqrt(prod(cube.ed-cube.st+1,2));
else
    weights = weights(:); % change to column vector
end
if isscalar(scalePenalties)
    scalePenalties = ones(length(cube.st),1).*scalePenalties;
end
if (nargin < 5) || isempty(ui)
    ui = v;
end
if (nargin < 6) || isempty(verbose)
    verbose = 0;
end
if (nargin < 7) || isempty(lambda)
    lambda   = 0.02;
end
if (nargin < 8) || isempty(maxit)
    maxit    = 1e3;
end

% default parameters
dif_tol  = 1e-5;
gap_tol  = 1e-5;

maxit_dyk          = 500; 
increment_tol_dyk  = 1e-5;
constraint_tol_dyk = 1e-3;

maxit_tv = 500;
tol_tv   = 1e-3;

% initialization
cit   = 0;
uo    = ui; 
w     = 0.*v;
p     = 0.*v;

normv = norm(v, 'fro'); 
gap   = norm(uo - v + w, 'fro'); 
done  = 0;
check = 50;

if nargout > 1
    stat = cell(maxit, 1);
end
if verbose > 0
    fprintf('MR-TV regression via ADMM ...\n');
    timeADMM = tic;
end

% ADMM iteration
while ~done
    cit = cit + 1;
    w   = projectionMLSball2d(lambda*p+v-uo, cube, weights, scalePenalties, ...
        maxit_dyk, increment_tol_dyk, constraint_tol_dyk, verbose-1);
    if verbose > 1
        fprintf('TV regression ...\n');
        timeTV = tic;
    end
    u = tvRegression2dPG(lambda*p+v-w, lambda, uo, maxit_tv, tol_tv, verbose-1);
    if verbose > 1
        fprintf('elapse %g\n', toc(timeTV));
    end
    p  = p - (u + w - v)/lambda;
    
    gap = norm(u + w - v, 'fro');
    dif = norm(u - uo, 'fro');
    if (cit >= maxit) || (dif <= dif_tol*normv && min(gap,gap/normv) <= gap_tol)
        done = 1;
    end
    if (mod(cit,check) == 0) && (verbose > 1)  
        fprintf('ADMM iteration %d\n', cit);
        fprintf('          gap to equality: %.5f\n', gap);
        fprintf('          size of increment (in image space): %.5f\n', dif);
    end
    if verbose > 2
        figure(1); clf;
        subplot(131); imagesc(v);   axis off; title('data');                      drawnow;
        subplot(132); imagesc(u);   axis off; title(sprintf('sol. it. %d', cit)); drawnow;        
        subplot(133); imagesc(v-u); axis off; title('residual');                  drawnow;
    end
    uo  = u;
    if nargout > 1
        stat{cit} = u;
    end
end

if nargout > 1
    stat = stat(1:cit);
end
if verbose > 0
    fprintf('elapse %g s\n', toc(timeADMM));
    fprintf('stop at: # iterations: %d / %d\n', cit, maxit);
    if verbose > 1
        fprintf('          gap to equality: %.5f\n', gap);
        fprintf('          size of increment: %.5f\n', dif);
    end
end

% Equivalent optimization problem
% 
%   minimize ||\nabla u||_1 + Chi(w)
%   subject to u + w - v = 0
%
% Here Chi(w) = { 0     if (w_B)^(-1) | < e_B, w > | - q_B <= 0
%               { inf   otherwise
%
% Algorithm: Alternating Direction Method of Multipliers (ADMM)
%   augmented Lagrangian
%        L_{lambda}(u, w, p) = ||\nabla u||_1 + Chi(w) ...
%               - 1/lambda <p, u + w - v> + 1/(2 lambda)||u + w - v||_{2}^2
%       
%   iterations
%       w := argmin_{w} L_{lambda}(u,w,p)
%       u := argmin_{u} L_{lambda}(u,w,p)
%       p := p - (u + w - v)