function [u, stat] = nemirovskiRegression1dCP(v, k, L, intval, ui, maxit, verbose)
%NEMIROVSKIREGRESSION1DCP Compute Nemirovski's estimator by Chambolle-Pock
%algorithm, i.e.
%
%   min || u - v ||_{B}
%   subject to || D^k u ||^2_{L^2} <= L^2
%
%Usage:
% [u, stat] = nemirovskiRegression1dCP(v, k, L, intval, ui, maxit, verbose)
% u = nemirovskiRegression1dCP(v, k, L, intval)
%
%Input:
%   v:       data
%   k:       Sobolev index
%   L:       threshold
%   intval:  system of intervals, a list of 'li' and 'ri'
%   ui:      initial value (default v)
%   maxit:   maximal number of iterations (default 1e3)
%   verbose: level of screen print (default 0)
%
%Output:
%   u:    the solution
%   stat: details of iterations

% Housen Li @ MPIbpc
% 16.01.2015 create the code
% 05.01.2016 change name "mrNemirovski1dCPspIntval" --> 
%            "nemirovskiRegression1dCP"

% default input
if nargin < 5
    ui = 0.*v;
end
if nargin < 6
    maxit = 1e3;
end
if nargin < 7
    verbose = 0;
end

% default parameters
tau   = 1;
sigma = 1; % parameter in Chambolle-Pock, not noise level
theta = 1;
check = 50;

if verbose > 0
    tic;
    fprintf('Nemirovski estimator via Chambolle-Pock algorithm (spline version)\n');
    fprintf('  Iter   Objective-Value   Constraint-Residual \n');
end

if nargout > 1
    stat    = cell(maxit, 1);
end

% initialization
x0   = ui(:);
xbar = x0;
% mrv  = mrcoef_mex(v(:), scl);
mrv  = mrcoefIntval(v, intval);
y    = 0.*mrv;
n    = length(x0);

% iterations
for i = 1:maxit
    if (verbose > 0) && (mod(i, check) == 0)
%         fval   = mr_constraint_residual(x, v, scl, 0, 0);
        fval   = mrstatIntval_mex(x(:)-v(:), intval.li(:)-1, intval.ri(:)-1);
        conRes = l2norm_der_sp(x, k) - L;
        fprintf('%6d   %15.8g        %10.8g\n', i, fval, conRes);
        if (mod(i, check*20) == 0) && (i > maxit)
            fprintf('\n  Iter   Objective-Value   Constraint-Residual\n');
        end
    end
    
%     Kxbar = mrcoef_mex(xbar, scl);
    Kxbar = mrcoefIntval(xbar, intval);
    y     = prox_Fstar(y + sigma*Kxbar, sigma, mrv, verbose-1);
    
%     Kstary = mrdual_mex(n, 1, y, scl);
    Kstary = mrdualIntval(n, y, intval);
    x      = prox_G(x0 - tau*Kstary, tau, k, L, verbose-1);
    
    xbar = x + theta*(x - x0);
    x0   = x;
    if nargout > 1
        stat{i} = x;
    end
end

u = x;

if verbose > 0
%     fval   = mr_constraint_residual(u, v, scl, 0, 0);
    fval   = mrstatIntval_mex(u(:)-v(:), intval.li(:)-1, intval.ri(:)-1);
    conRes = l2norm_der_sp(u, k) - L;
    fprintf('Chambolle-Pock algorithm used %d iterations\n', maxit);
    fprintf('Objective-Value is     %10.8g\n', fval);
    fprintf('Constraint-Residual is %10.8g\n', conRes);
    fprintf('elapses                %10.8g seconds\n', toc);
end

end

% subroutines

function [ y ] = prox_Fstar(y0, sigma, mrv, verbose)

y0 = y0 - sigma*mrv;

y  = y0 - sigma * prox_inf_norm(y0/sigma, 1/sigma);

end

function [ x ] = prox_G(x0, tau, k, L, verbose)

tol_eps = 1e-10;

x0  = x0 - mean(x0);
fx0 = fft(x0);

n = numel(fx0);

% B-spline bases matrix
t = linspace(0, 1, n+1);
t = t(1:end-1);
a = nbspline(t(:), 2*k, 0, n);
a = a(:);
% Q = sparse(toeplitz(a, a([1, end:-1:2])));
fQ  = fft(a);
fQt = fft(a([1, end:-1:2]));
fQ(abs(fQ) < tol_eps)   = 1;
fQt(abs(fQt) < tol_eps) = 1;

% finite difference matrix
fD  = fft([n; -n; zeros(n-2,1)]).^k;
fDt = fft([n; zeros(n-2,1); -n]).^k;

% Gramian matrix
a = zeros(1, n);
f0 = @(par) nbspline(par, k, 0, n);
for i = -(k-1):(k-1)
    fi = @(par) nbspline(par, k, i, n);
    ip = @(par) fi(par) .* f0(par);
    a(mod(i, n)+1) = quadgk(ip, 0, 1);
end
% G = sparse(toeplitz(a));
fG = fft(a(:));

w = real(fDt ./ fQt .* fG .* fD ./ fQ) / n;

% fast but restrict to diagonal matrices
fx = proj_ellipsoidvar([real(fx0);imag(fx0)], [w(:); w(:)], L^2, verbose);

x = real(ifft(fx(1:n)+1i*fx((n+1):(2*n))));

x = x - mean(x);

end