function [ u, alphaL, stat ] = penmindLepskiiRegression1dQP( v, k, intval, sd, q, kappa, alpha0, C0, verbose )
%MRTIKHONOVLEPSKII1D penMIND with Lepskii principle for one-dimensional
%regression via function penmindRegression1dQP, see (Li, 2016, Section 2.5) 
%
%Usage:
%   [u, alphaL, stat] = penmindLepskiiRegression1dQP(v, k, intval, sd, ...
%                                            q, kappa, alpha0, C0, verbose)
%   u = penmindLepskiiRegression1dQP(v, k, intval, sd) 
%
%Input:
%   v: data
%   k: Sobolev index
%   intval: system of intervals, a list of 'li' and 'ri'
%   sd:      standard deviation of noise
%   q:       Lq-risk (default 2)
%   kappa:   parameter for Lepskii principle (default 1.1)
%   alpha0:  parameter for Lepskii principle (default 0.02*sd)
%   C0:      parameter for Lepskii principle (default 0.45)
%   verbose: level of screen print and plot  (default 0)
%
%Output:
%   u: the solution
%   alphaL: penalization parameter by Lepskii principle
%   stat:   details of computation, see function penmindRegression1dQP

% Housen Li @ MPIbpc
% 13.12.2015 create the code
% 05.01.2016 change name "mrTikhonovLepskii1d" --> 
%            "penmindLepskiiRegression1dQP"

% default input
if (nargin < 5) || isempty(q)
    q = 2;
end
if (nargin < 6) || isempty(kappa)
    kappa = 1.1;
end
if (nargin < 7) || isempty(alpha0)
    alpha0 = 0.02*sd;
end
if (nargin < 8) || isempty(C0)
    C0 = 0.40; % 0.45;
end
if nargin < 9
    verbose = 0;
end

% default parameters
maxNumAlpha = 200;
if q <= 4*k+2
    theta = k/(2*k + 1);
else
    theta = (k-1/2+1/q)/(2*k);
end
thetaPrime = theta * 2 * k;    

n = numel(v);

% Lepskii
recUcand    = cell(maxNumAlpha, 1);
rad_lepskii = zeros(maxNumAlpha, 1);
diffLq      = zeros(maxNumAlpha, 1);

if verbose > 0
    fprintf('i = 1 / %d\n', maxNumAlpha);
end
alpha          = alpha0;
isDone         = 0;
recUcand{1}    = penmindRegression1dQP(v, k, intval, alpha, verbose-1);
rad_lepskii(1) = 4*C0*maxPSI(alpha, n, sd, theta, thetaPrime);
for i = 2:maxNumAlpha
    if verbose > 0 
        fprintf('i = %d / %d\n', i, maxNumAlpha);
    end
    alpha = alpha * kappa;
    uCand = penmindRegression1dQP(v, k, intval, alpha, verbose-1);
    rad_lepskii(i) = 4*C0*maxPSI(alpha, n, sd, theta, thetaPrime);
    for j = 1:(i-1)
        diffLq(j) = norm(uCand - recUcand{j},q)/(n^(1/q));
        if diffLq(j) > rad_lepskii(j)
            isDone = 1;
            break;
        end   
    end
    if isDone == 1
        alphaL = alpha/kappa;
        u      = recUcand{i-1};
        break;
    else
        recUcand{i} = uCand;
    end
end

if isDone == 0 
    fprintf('Maximal candidate (alpha = %g) is reached!\n', alpha);
    fprintf('The return estimate is the one with maximal alpha.\n');
    alphaL = alpha;
    u      = recUcand{end};
end

stat.diffLq      = diffLq;
stat.rad_lepskii = rad_lepskii;

if verbose > 0
    fprintf('Lepskii parameters: \n');
    fprintf(' risk order            q = %g\n', q);
    fprintf(' start at         alpha0 = %g\n', alpha0);
    fprintf(' increment factor  kappa = %g\n', kappa);
    fprintf(' max number  maxNumAlpha = %g\n', maxNumAlpha);
    fprintf(' constant             C0 = %g\n', C0);
end

end

function [ maxVal ] = maxPSI(alpha, n, sd, theta, thetaPrime) 

psiA = (sd^2*log(n))^(0.25+0.5*theta)/n^(theta)/alpha^(0.5-theta);

psiB = sd * sqrt(log(n) / n);

psiC = (sd^2*log(n))^(0.25)/sqrt(alpha)/n^(thetaPrime);

maxVal = max([psiA, psiB, psiC]);

end

