function [ u, stat ] = penmindRegression1dQP( v, k, intval, penCoef, verbose, algorithm )
%PENMINDREGRESSION1DQP Solve multiresolution Tikhonov regularization
%with Sobolev penalty for one-dimensional regression by built-in function 
%quadprog, i.e. 
%
%Optimization
%   min_u      || u - v ||_B + penCoef/2 || D^k u ||_2^2
%   subject to mean(u) = 0
%
%Equivalent formulation
%   min_{u,w}  w + penCoef/2 || D^k u ||_2^2
%   subject to || u - v ||_B <= w
%              mean(u) = 0
%Usage:
% [u, stat] = penmindRegression1dQP(v, k, intv, lambda, verbose, algorithm)
% u = penmindRegression1dQP(v, k, intv, lambda)
%Input:
%   v:         data
%   k:         Sobolev index
%   intval:    system of intervals, a list of 'li' and 'ri'
%   penCoef:   penalization coefficient 
%   verbose:   level of screen print
%   algorithm: algorithm for 'quadprog' (interior point method by default)
%
%Output:
%   u:        the solution         
%   stat:     statistics for iterations
%       fval:     value of objective function
%       exitflag: integer identifying the reason the algorithm terminated
%       output:   structure containing information about the optimization
%       lambda:   structure containing the Lagrange multipliers at the 
%                 solution
%   see 'quadprog' for further details about the last four return values



% Housen Li @ MPIbpc
% 13.12.2015 create the code
% 05.01.2016 change name "mrTikhonovSobolevReg1dQP" --> 
%            "penmindRegression1dQP"
%            add variable 'algorithm'

% default input
if nargin < 5
    verbose = 1;
end
switch verbose
    case 0
        display_level = 'off';
    case 1
        display_level = 'final';
    case 2
        display_level = 'iter';
    case 3
        display_level = 'iter-detailed';
    case 4
        display_level = 'final-detailed';
    otherwise
        fprintf('Use the max verbose (4)!\n');
        display_level = 'final-detailed';
end
if nargin < 6
    algorithm = 'interior-point-convex';
end
        
% default parameters
tolfun    = 1e-10;
tolcon    = 1e-10;
tolx      = 1e-10;
maxit     = 200;

sz = size(v);
v  = v(:);
n  = numel(v);
ni = length(intval.li);

% B-spline bases matrix
x = linspace(0, 1, n+1);
x = x(1:end-1);
a = nbspline(x(:), 2*k, 0, n);
Q = sparse(toeplitz(a, a([1, end:-1:2])));

% Gramian matrix
a = zeros(1, n);
f0 = @(par) nbspline(par, k, 0, n);
for i = -(k-1):(k-1)
    fi = @(par) nbspline(par, k, i, n);
    ip = @(par) fi(par) .* f0(par);
    a(mod(i, n)+1) = quadgk(ip, 0, 1);
end
G = sparse(toeplitz(a));
H = blkdiag(sparse(n, n), G, 0);
H = 0.5*(H + H');

D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
D(1,n) = -1;
D      = sqrt(n)^k * D^k;
% D      = n^k * D^k;

% inequality matrix
B = zeros(2*ni, n);
for i = 1:ni
    len = 1/sqrt(intval.ri(i) - intval.li(i) + 1);
    B(i,   intval.li(i):intval.ri(i)) = len;
    B(i+ni,intval.li(i):intval.ri(i)) = -len;
end
B = sparse(B);
b = B * v;
B = B * Q;
B = sparse([B, sparse(2*ni,n), -ones(2*ni,1)]);
Aeq = sparse([ones(1,2*n), 0; D, -speye(n,n+1)]);
beq = zeros(n+1,1);

opts = optimoptions('quadprog', 'Algorithm', algorithm, 'Display', display_level, 'Diagnostics', 'off', ...
    'TolFun', tolfun, 'TolCon', tolcon, 'TolX', tolx, 'MaxIter', maxit);

% initial value
x0  = zeros(2*n+1, 1);

if verbose > 0 
    fprintf('Start quadratic programming ...\n'); tic;
end

% quadratic programming
[c, fval, exitflag, output, lambda] = ...
    quadprog(H, [zeros(2*n,1); 2/penCoef], B, b, Aeq, beq, [], [], x0, opts);

if verbose > 0 
    fprintf('... elapse %g s\n', toc);
end

u = Q * c(1:n);
u = reshape(u, sz);

% additional information
stat.fval     = fval;
stat.exitflag = exitflag;
stat.output   = output;
stat.lambda   = lambda;

end

