function [ u ] = projectionMLSball1d( ui, intv, weights, scalePenalties, ...
    max_it, increment_tol, constraint_tol, verbose )
%PROJECTIONMLSBALL1D Projection on the Multiscale Linear Statistics (MLS)
%"ball" in 1-dimension, i.e. the solution of
%
%   min || u - ui ||^2
%   s.t. (w_B)^(-1) | < e_B, u > | - q_B <=0  for every B
%
%using Dykstra's algorithm
%
%Usage
%   u = projectionMLSball2d(ui,cube,weights,scalePenalties,...
%                           max_it,increment_tol,constraint_tol,verbose)
%   u = projectionMLSball2d(ui,cube,[],scalePenalties)
%
%Input
%   ui                a matrix, data
%   intv              a list (li, ri), li and ri are nIntv x 1 vectors
%   weights           a vector, weights (by default sqrt(#samples))
%   scalePenalties    a vector, scale penalties (incorporating thresholds)
%   max_it            an integer, maximal number of Dykstra's iterations
%   increment_tol     a scalar, tolerance of increments between two
%                     consecutive iterations
%   constraint_tol    a scalar, tolerance of constraint
%   verbose           level of screen print
%                     0 - print nothing
%                     1 - print a brief summary 
%                     2 - print details of iterations
%
%Output
%   u                 the solution
    
% Housen Li @ MPIbpc
% 10.12.2015 create the code

if isempty(weights)
    weights = sqrt(intv.ri-intv.li+1);
else
    weights = weights(:); % change to column vector
end
if (nargin < 4) || isempty(scalePenalties)
    error('The scale penalties have to be specified!')
end
if isscalar(scalePenalties)
    scalePenalties = scalePenalties * ones(size(weights));
else
    scalePenalties = scalePenalties(:);
end
if (nargin < 5) || isempty(max_it)
    max_it = 5e3;
end
if (nargin < 6) || isempty(increment_tol)
    increment_tol = 1e-5;
end
if (nargin < 7) || isempty(constraint_tol)
    constraint_tol = 1e-5;
end
if nargin < 8
    verbose = 0;
end

if verbose > 0
    tic;
    fprintf('*********************************************************\n');
    fprintf('Projection on MLS ''ball'' via Dykstra (d = 1) ...\n');
end

% C implementation
u = projectionMLSball1d_mex(double(ui), intv.li-1, intv.ri-1, weights, ...
    scalePenalties, max_it, increment_tol, constraint_tol, verbose);
if verbose > 0 
    fprintf('elapse %g s\n', toc);
    fprintf('*********************************************************\n');
end

