/* H. Li - December 10, 2015 */

#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "multiresolution1d.c"

double projectSingleIntv(double*, const double*, int, int, int, double);

/* u = projectionMLSball1d_mex(ui, li, ri, weights, scalePenalties, max_it, increment_tol, constraint_tol, verbose); */
/*      ui:  input, column vector                                                                                    */
/*      li:  left end indices of input intervals, start from 0, column vector                                        */
/*      ri:  right end indices of input intervals, start from 0, column vector                                       */
/*                                                                                                                   */
/* verbose 0: print nothing                                                                                          */
/*         1: print statistics at the end of iterations                                                              */
/*         2: print above, differences of each step, and gap when we check                                           */
/*         3: print above, and gap of each step                                                                      */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input */
    const double *ui = mxGetPr(prhs[0]);
    const int      n = mxGetM(prhs[0]);
    
    const double *li    = mxGetPr(prhs[1]);
    const double *ri    = mxGetPr(prhs[2]);
    const int     nIntv = mxGetM(prhs[1]);
    
    const double *weights        = mxGetPr(prhs[3]);
    const double *scalePenalties = mxGetPr(prhs[4]);
    
    const int    max_it          = (int)mxGetScalar(prhs[5]);
    const double increment_tol   = mxGetScalar(prhs[6]);
    const double constraint_tol  = mxGetScalar(prhs[7]);
    const int    verbose         = (int)mxGetScalar(prhs[8]);
    
    const double eps_tol         = 1e-16;
    const int    check           = 50;
    
    /* output */
    plhs[0]   = mxCreateDoubleMatrix(n, 1, mxREAL);
    double *u = mxGetPr(plhs[0]);
    
    if (verbose > 0) {
        mexPrintf("Dykstra parameters: ");
        mexPrintf("max_it = %d, increment_tol = %g, constraint_tol = %g\n", max_it, increment_tol, constraint_tol);
    }
    
    
    /* Dykstra's projection algorithm */
    /*      initialization */
    double magnitude0 = 0.;
    for (int k = 0; k < n; ++k) {
        u[k]  = ui[k];
        magnitude0 = magnitude0 + fabs(u[k]);
    }
    magnitude0 = magnitude0 / n;
    int nres = 0;
    for (int k = 0; k < nIntv; ++k) {
        nres = nres + (int)ri[k] - (int)li[k] + 1;
    }
    double *res = (double *)mxCalloc(nres, sizeof(double));
    double *v   = (double *)mxMalloc(n * sizeof(double));
    
    /*      iteration */
    int nit  = 0;
    int done = 0;
    int cnt, lii, rii;
    double increment, constraintGap;
    while (done == 0) {
        nit = nit + 1;
        cnt = 0;
        increment = 0.;
        for (int k = 0; k < nIntv; ++k) {
            lii = (int)li[k];
            rii = (int)ri[k];
            /* compute difference */
            for (int i = lii; i <= rii; ++i) {
                v[i] = u[i] - res[cnt+(i-lii)];
            }
            /* projection on each interval */
            increment = fmax(increment, projectSingleIntv(u, v, n, lii, rii, weights[k]*scalePenalties[k]));
            /* update residuals */
            for (int i = lii; i <= rii; ++i) {
                res[cnt+(i-lii)] = u[i] - v[i];
            }
            cnt = cnt + (rii - lii + 1);
        }
        
        /* stop criteria */
        increment = increment / fmax(magnitude0, eps_tol);
        if (increment < increment_tol) {
            constraintGap = mrLinearStat1d(u, n, li, ri, nIntv, weights, scalePenalties);
            if (constraintGap < constraint_tol) {
                done = 2;
                break;
            } else if (verbose > 1) {
                mexPrintf(" ... It. %d, increment = %g, current constraint gap = %g\n", nit, increment, constraintGap);
            }
        } else if ((verbose > 1) && (nit%check == 0)) {
            mexPrintf(" ... It. %d, increment (max over pixels and current cyclic loop) = %g\n", nit, increment);
        }
        if (nit >= max_it) {
            done = 1;
            break;
        }
    }
    if (verbose > 0) {
        mexPrintf("Dykstra stopped at\n");
        if (done == 1) {
            mexPrintf("    maximal no. of iteration is reached\n");
            constraintGap = mrLinearStat1d(u, n, li, ri, nIntv, weights, scalePenalties);
        } else {
            mexPrintf("    stopping criteria is satisfied\n");
        }
        mexPrintf("    no. of interation = %d\n", nit);
        mexPrintf("    size of increment = %f\n", increment);
        mexPrintf("    gap of constraint =  %f\n", constraintGap);
    }
    mxFree(res);
    mxFree(v);
    return;
}

/* ---------- projection on a single interval constraint */
double projectSingleIntv(double *uo,         /* output n x 1 (only values in the interval are modified) */
                    const double *ui, int n, /* input n x 1 (only values in the interval are used) */
                    int lii, int rii,        /* specifying cube */
                    double threshold)        /* threshold */
/* return the maximal difference of uo (input) and uo (output) over pixels */
{
    double locSum = 0; /* local sum */
    for (int i = lii; i <= rii; ++i) {
        locSum = locSum + ui[i];
    }
    
    double res;
    if (fabs(locSum) <= threshold) {
        res = 0;
    } else {
        res = (threshold * sgn(locSum) - locSum) / (rii - lii + 1.);
    }
    double diff = 0.; /* maximal difference to previous estimate */
    double u_aux;
    for (int i = lii; i <= rii; ++i) {
        u_aux = uo[i];
        uo[i] = ui[i] + res;
        diff  = fmax(diff, fabs(uo[i]-u_aux));
    }
    return diff;
}