function [ q, stats ] = quantileMLS1d( n, interval, weights, scalePenalties, alpha, nsim, Kh )
%QUANTILEMLS1D Quantile of Multiscale Linear Statistics (MLS)
%
%via Monte-Carlo simulations
%
%Usage
%   [q,stats] = quantileMLS1d(n,interval,weights,scalePenalties,alpha,nsim,Kh)
%   q         = quantileMLS1d(n,interval,[],[],alpha)
%
%Input
%   n               a scalar, number of samples
%   interval        a list (li, ri), li and ri are nInterval x 1 vectors
%   weights         a vector, weights (by default sqrt(#samples))
%   scalePenalties  a vector, scale penalties (by defaul 0)
%   alpha           a scalar in [0, 1], significance level
%   nsim            an integer, number of simulations
%   Kh              a function handler
%
%Output
%   q               a scalar, alpha lower quantile of MLS
%   stats           a vector, record MLS for each simulation

% Housen Li @ MPIbpc
% 09.25,2015 create the code
% 06.10.2015 extend to w_I^{-1} | < e_I, K' xi > | - q_I
% 09.11.2015 build from 'quantileMLS2d.m'

if (nargin < 3) || (isempty(weights))
    weights = sqrt(interval.ri-interval.li+1);
end
if (nargin < 4) || isempty(scalePenalties)
    scalePenalties = zeros(length(weights),1);
end 
if nargin < 5
    alpha = 0.9;
end
if nargin < 6
    nsim = 1e4;
end
if nargin < 7
    Kh = @(par) par;
end

% check validity
if length(Kh(randn(n,1))) < max([interval.li(:);interval.ri(:)]) 
    error('Some interval lies outside the domain!');
end
    

fprintf('Simulate %g lower quantile of Multiscale Linear Statistic (MLS) ...\n', alpha);
tm = tic;
stats = zeros(nsim,1);
for i = 1:nsim
    xi       = Kh(randn(n,1));
    stats(i) = mrLinearStat1d_mex(xi(:), interval.li(:)-1, interval.ri(:)-1, weights(:), scalePenalties(:));
end
q = quantile(stats, alpha);
fprintf('Number of simulations is %d, elapse %g s\n', nsim, toc(tm));

