function [ u ] = projectionMLSball2d( ui, cube, weights, scalePenalties, ...
    max_it, increment_tol, constraint_tol, verbose )
%PROJECTIONMLSBALL2D Projection on the Multiscale Linear Statistics (MLS)
%"ball" in 2-dimension, i.e. the solution of
%
%   min || u - ui ||^2
%   s.t. (w_B)^(-1) | < e_B, u > | - q_B <=0  for every B
%
%using Dykstra's algorithm
%
%Usage
%   u = projectionMLSball2d(ui,cube,weights,scalePenalties,...
%                           max_it,increment_tol,constraint_tol,vis_level)
%   u = projectionMLSball2d(ui,cube,[],scalePenalties)
%
%Input
%   ui                a matrix, data
%   cube              a list (st, ed), st and ed are nCube x 2 matrices
%   weights           a vector, weights (by default sqrt(#samples))
%   scalePenalties    a vector, scale penalties (incorporating thresholds)
%   max_it            an integer, maximal number of Dykstra's iterations
%   increment_tol     a scalar, tolerance of increments between two
%                     consecutive iterations
%   constraint_tol    a scalar, tolerance of constraint
%   verbose           level of screen print
%                     0 - print nothing
%                     1 - print a brief summary 
%                     2 - print details of iterations
%
%Output
%   u                 the solution
    
% Housen Li @ MPIbpc
% 30.09.2015 create the code

if isempty(weights)
    weights = sqrt(prod(cube.ed-cube.st+1,2));
else
    weights = weights(:); % change to column vector
end
if (nargin < 4) || isempty(scalePenalties)
    error('The scale penalties have to be specified!')
else
    scalePenalties = scalePenalties(:);
end
if (nargin < 5) || isempty(max_it)
    max_it = 5e3;
end
if (nargin < 6) || isempty(increment_tol)
    increment_tol = 1e-5;
end
if (nargin < 7) || isempty(constraint_tol)
    constraint_tol = 1e-5;
end
if nargin < 8
    verbose = 0;
end

if verbose > 0
    tic;
    fprintf('*********************************************************\n');
    fprintf('Projection on MLS ''ball'' via Dykstra ...\n');
end

% C implementation
u = projectionMLSball2d_mex(double(ui), cube.st-1, cube.ed-1, weights, ...
    scalePenalties, max_it, increment_tol, constraint_tol, verbose);
if verbose > 0 
    fprintf('elapse %g s\n', toc);
    fprintf('*********************************************************\n');
end

