/* H. Li - September 30, 2015 */

#include "matrix.h"
#include "mex.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "multiresolution2d.c"

double projectSingleCube(double*, const double*, int, int, int, int, int, int, double);

/* u = projectionMLSball2d_mex(ui, st, ed, weights, scalePenalties, max_it, increment_tol, constraint_tol, verbose); */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input */
    const double *ui             = mxGetPr(prhs[0]);
    const int     m              = mxGetM(prhs[0]);
    const int     n              = mxGetN(prhs[0]);
    const double *st             = mxGetPr(prhs[1]);
    const double *ed             = mxGetPr(prhs[2]);
    const int     nCube          = mxGetM(prhs[1]);
    const double *weights        = mxGetPr(prhs[3]);
    const double *scalePenalties = mxGetPr(prhs[4]);
    const int    max_it          = (int)mxGetScalar(prhs[5]);
    const double increment_tol   = mxGetScalar(prhs[6]);
    const double constraint_tol  = mxGetScalar(prhs[7]);
    const int    verbose         = (int)mxGetScalar(prhs[8]);
    
    const double eps_tol         = 1e-16;
    const int    check           = 50;
    
    /* output */
    plhs[0]   = mxCreateDoubleMatrix(m, n, mxREAL);
    double *u = mxGetPr(plhs[0]);
    
    if (verbose > 0) {
        mexPrintf("Dykstra parameters: ");
        mexPrintf("max_it = %d, increment_tol = %g, constraint_tol = %g\n", max_it, increment_tol, constraint_tol);
    }
    
    /* Dykstra's projection algorithm */
    /*      initialization */
    double magnitude0 = 0.;
    for (int k = 0; k < m*n; ++k) {
        u[k]  = ui[k];
        magnitude0 = magnitude0 + fabs(u[k]);
    }
    magnitude0 = magnitude0 / (m*n);
    int nres = 0;
    for (int k = 0; k < nCube; ++k) {
        nres += ((int)ed[k] - (int)st[k] + 1)*((int)ed[k+nCube] - (int)st[k+nCube] + 1);
    }
    double *res = (double *)mxCalloc(nres, sizeof(double));
    double *v   = (double *)mxMalloc(m*n*sizeof(double));
    
    /*      iteration */
    int nit  = 0;
    int done = 0;
    int cnt, numr;
    int str, edr, stc, edc;
    double increment, constraintGap;
    while (done == 0) {
        nit = nit + 1;
        cnt = 0;
        increment = 0.;
        for (int k = 0; k < nCube; ++k) {
            str = (int)st[k];
            edr = (int)ed[k];
            stc = (int)st[k+nCube];
            edc = (int)ed[k+nCube];
            /* compute difference */
            numr = edr - str + 1;
            for (int j = stc; j <= edc; ++j) {
                for (int i = str; i <= edr; ++i) {
                    v[i+j*m] = u[i+j*m] - res[cnt+(i-str)+(j-stc)*numr];
                }
            }
            /* projection on each cube */
            increment = fmax(increment, projectSingleCube(u, v, m, n, str, edr, stc, edc, weights[k]*scalePenalties[k]));
            /* update residuals */
            for (int j = stc; j <= edc; ++j) {
                for (int i = str; i <= edr; ++i) {
                    res[cnt+(i-str)+(j-stc)*numr] = u[i+j*m] - v[i+j*m];
                }
            }
            cnt += numr * (edc - stc + 1);
        }
        /* stop criteria */
        increment = increment / fmax(magnitude0, eps_tol);
        if (increment < increment_tol) {
            constraintGap = mrLinearStat2d(u, m, n, st, ed, nCube, weights, scalePenalties);
            if (constraintGap < constraint_tol) {
                done = 2;
                break;
            } else if (verbose > 1) {
                mexPrintf(" ... It. %d, increment = %g, current constraint gap = %g\n", nit, increment, constraintGap);
            }
        } else if ((verbose > 1) && (nit%check == 0)) {
            mexPrintf(" ... It. %d, increment (max over pixels and current cyclic loop) = %g\n", nit, increment);
        }
        if (nit >= max_it) {
            done = 1;
            break;
        }
    }
    if (verbose > 0) {
        mexPrintf("Dykstra stopped at\n");
        if (done == 1) {
            mexPrintf("    maximal no. of iteration is reached\n");
            constraintGap = mrLinearStat2d(u, m, n, st, ed, nCube, weights, scalePenalties);
        } else {
            mexPrintf("    stopping criteria is satisfied\n");
        }
        mexPrintf("    no. of interation = %d\n", nit);
        mexPrintf("    size of increment = %f\n", increment);
        mexPrintf("    gap of constraint =  %f\n", constraintGap);
    }
    mxFree(res);
    mxFree(v);
    return;
}


int sgn(double a) { return (a == 0) ? 0 : (a < 0 ? -1 : 1); }
/* ---------- projection on a single cube constraint (also works for rectangles) */
double projectSingleCube(double * uo,                        /* output m x n (only values in the cube are modified) */
                       const double *ui, int m, int n,     /* input m x n (only values in the cube are used) */
                       int str, int edr, int stc, int edc, /* specifying the cube */
                       double threshold)                   /* threshold */
/* return the maximal difference of uo (input) and uo (output) over pixels */
{
    double locSum = 0.; /* local sum */
    for (int j = stc; j <= edc; ++j) {
        for (int i = str; i <= edr; ++i) {
            locSum += ui[i+j*m];
        }
    }
    double res;
    if (fabs(locSum) <= threshold) {
        res = 0.;
    } else {
        res = (threshold * sgn(locSum) - locSum) / ((edr-str+1.)*(edc-stc+1.));
    }
    double diff   = 0.; /* maximal difference to previous estimate */
    double u_aux;
    for (int j = stc; j <= edc; ++j) {
        for (int i = str; i <= edr; ++i) {
            u_aux     = uo[i+j*m];
            uo[i+j*m] = ui[i+j*m] + res;
            diff      = fmax(diff, fabs(uo[i+j*m]-u_aux));
        }
    }
    return diff;
}

