function [ q, stats ] = quantileMLS2d( sz, cube, weights, scalePenalties, alpha, nsim, Kh )
%QUANTILEMLS2D Quantile of Multiscale Linear Statistics (MLS)
%
%via Monte-Carlo simulations
%
%Usage
%   [q,stats] = quantileMLS2d(sz,cube,weights,scalePenalties,alpha,nsim,Kh)
%   q        = quantileMLS2d(sz,cube,[],[],alpha)
%
%Input
%   sz              a 2-vector, size of domain
%   cube            a list (st, ed), st and ed are nCube x 2 matrices
%   weights         a vector, weights (by default sqrt(#samples))
%   scalePenalties  a vector, scale penalties (by defaul 0)
%   alpha           a scalar in [0, 1], significance level
%   nsim            an integer, number of simulations
%   Kh              a function handler
%
%Output
%   q               a scalar, alpha lower quantile of MLS
%   stats           a vector, record MLS for each simulation

% Housen Li @ MPIbpc
% 09.25,2015 create the code
% 06.10.2015 extend to w_b^{-1} | < e_B, K' xi > | - q_B

if numel(sz) < 2
    error('The 1st argument should be a 2-vector for the domain size!');
end
if (nargin < 3) || (isempty(weights))
    weights = sqrt(prod(cube.ed-cube.st+1,2));
else
    weights = weights(:); % change to column vector
end
if (nargin < 4) || isempty(scalePenalties)
    scalePenalties = zeros(length(weights),1);
else
    scalePenalties = scalePenalties(:);
end 
if nargin < 5
    alpha = 0.9;
end
if nargin < 6
    nsim = 1e4;
end
if nargin < 7
    Kh = @(par) par;
end

% check validity
if (size(Kh(randn(sz)), 1) < max([cube.st(:,1);cube.ed(:,1)])) ...
        || (size(Kh(randn(sz)), 2) < max([cube.st(:,2);cube.ed(:,2)])) 
    error('Some cube lies outside the domain!');
end
    

fprintf('Simulate %g lower quantile of Multiscale Linear Statistic (MLS) ...\n', alpha);
tm = tic;
stats = zeros(nsim,1);
for i = 1:nsim
    xi       = Kh(randn(sz));
    stats(i) = mrLinearStat2d_mex(xi, cube.st-1, cube.ed-1, weights, scalePenalties);
end
q = quantile(stats, alpha);
fprintf('Number of simulations is %d, elapse %g s\n', nsim, toc(tm));

