function [ u ] = sobolevInversion1d(v, K, Kh, lambda, k, ui, maxit, tol, verbose)
%SOBOLEVINVERSION1D L2-norm as data fidelity and homogeneous Sobolev norm
%as regularization, i.e.
%
%   || Km u -v ||^2 + lambda || D^k u||^2
%
%Usage:
%   u = sobolevInversion1d(v, K, Kh, lambda, k, ui, maxit, tol, verbose)
%   u = sobolevInversion1d(v, K, Kh, lambda, k)
%
%Input:
% v:      data
% K:      forward operator (function handle or matrix)
% Kh:     adjoint operator (function handle or matrix)
% lambda: regularization parameter
% k:      sobolev index
% ui:     initial value
%
%Output:
% u:      solution (defined on [0, 1) and discretized on uniform grids)
%
%
%Algorithm: conjugate gradient (CG) method

% H. Li @ MPIbpc
% 16.12.2015 create the code

% default input
if ~isa(K, 'function_handle')
    K = @(par) K * par;
end
if ~isa(Kh, 'function_handle')
    Kh = @(par) Kh * par;
end
if (nargin < 6) || isempty(ui)
    ui = Kh(0.*v);
end
if (nargin < 7) || isempty(maxit)
    maxit = 1e4;
end
if (nargin < 8) || isempty(tol)
    tol = 1e-4;
end
if (nargin < 9) || isempty(verbose)
    verbose = 0;
end
    
% initialization
n      = length(ui);
D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
D(1,n) = -1;
Dk     = D^k.*n^k;
DkpDk  = (Dk'*Dk);

u      = ui;
r      = Kh(v) - (Kh(K(u))+lambda*DkpDk*u);
p      = r;
rsnull = r(:)'*r(:);
rsold  = rsnull;

if verbose > 0
    fprintf('Sobolev inversion ...\n');
end
if verbose > 1
    tic; fprintf('     original residual = %g\n', rsnull);
end

% it is already good
if sqrt(rsold) < tol
    if verbose > 0
        fprintf('     stop at: no iteration is needed\n');
    end
    return;
end

% CG iteration
for i = 1:maxit
    Ap    = Kh(K(p)) + lambda*DkpDk*p;
    alpha = rsold/(p(:)'*Ap(:));
    u     = u + alpha*p;
    r     = r - alpha*Ap;
    rsnew = r(:)'*r(:);
    if sqrt(rsnew/rsnull) < tol
%     if sqrt(rsnew) < tol
        break;
    end
    p     = r + rsnew/rsold*p;
    rsold = rsnew;
    if (verbose > 2) && (mod(i,50) == 0) 
        fprintf('     it = %d, residual = %g\n', i, rsnew);
    end
end

if verbose > 0
    fprintf('     stop at: # iterations = %d\n', i);
    if verbose > 1
        fprintf('     residual = %g\n', rsnew);
        fprintf('elapse %g\n', toc);
    end
end

end

