function [ u ] = sobolevInversion2d(v, K, Kh, lambda, k, ui, maxit, tol, verbose)
%SOBOLEVINVERSION2D Sobolev inversion: L2-norm as data misfit penalty with
%homogeneous Sobolev norm as regularization, i.e.
% 
%   minimize 1/2 ||K u - v||^2 + lambda/2 ||D^k u||^2
%
%Usage:
%   u = sobolevInversion2d(v, K, Kh, lambda, k, ui, maxit, tol, verbose)
%   u = sobolevInversion2d(v, K, Kh, lambda, k)
%
%Input:
% v:      data
% K:      forward operator (function handle or matrix)
% Kh:     adjoint operator (function handle or matrix)
% lambda: regularization parameter
% k:      sobolev index
% ui:     initial value
%
%Output:
% u:      solution (defined on [0, 1) and discretized on uniform grids)
%
%
%Algorithm: conjugate gradient (CG) method

% Housen Li @ MPIbpc
% 20.10.2014 create the code
% 08.01.2016 change name "sobolevInv" --> "sobolevInversion2d"

% default input
if ~isa(K, 'function_handle')
    K = @(par) K * par;
end
if ~isa(Kh, 'function_handle')
    Kh = @(par) Kh * par;
end
if (nargin < 6) || isempty(ui)
    ui = Kh(0.*v);
end
if (nargin < 7) || isempty(maxit)
    maxit = 500;
end
if (nargin < 8) || isempty(tol)
    tol = 1e-3;
end
if (nargin < 9) || isempty(verbose)
    verbose = 0;
end

% initialization
[m, n] = size(ui);
wr     = repmat(2*pi*circshift((-floor(m/2):1:(ceil(m/2)-1))', mod(m,2)), [1,n]);
wc     = repmat(2*pi*circshift(-floor(n/2):1:(ceil(n/2)-1), [0,mod(n,2)]), [m,1]);
w      = fftshift((wr.^2 + wc.^2).^k);

u      = ui;
r      = Kh(v) - (Kh(K(u))+lambda*real(ifft2(w.*fft2(u))));
p      = r;
rsnull = r(:)'*r(:);
rsold  = rsnull;

if verbose > 0
    fprintf('Sobolev inversion ...\n');
end
if verbose > 1
    tic;
    fprintf('     original residual = %g\n', rsnull);
end

% it is already good
if sqrt(rsold) < tol
    if verbose > 0
        fprintf('     stop at: no iteration is needed\n');
    end
    return;
end

% CG iteration
for i = 1:maxit
    Ap    = Kh(K(p)) + lambda*real(ifft2(w.*fft2(p)));
    alpha = rsold/(p(:)'*Ap(:));
    u     = u + alpha*p;
    r     = r - alpha*Ap;
    rsnew = r(:)'*r(:);
    if sqrt(rsnew/rsnull) < tol
%     if sqrt(rsnew) < tol
        break;
    end
    p     = r + rsnew/rsold*p;
    rsold = rsnew;
    if (verbose > 2) && (mod(i,50) == 0) 
        fprintf('     it = %d, residual = %g\n', i, rsnew);
    end
end

if verbose > 0
    fprintf('     stop at: # iterations = %d\n', i);
    if verbose > 1
        fprintf('     residual = %g\n', rsnew);
        fprintf('elapse %g\n', toc);
    end
end

end

