function [ u ] = tvInversion2dADMM( f, K, Kh, alpha, iterate_param, visual_param )
%TVINVERSION2DADMM L2-norm as data fidelity term and TV seminorm as
%regularization term, i.e.
%
%   minimize 1/2 ||K u - f||^2 + alpha ||\nabla u||_1
%
%Usage:
% u = tvInversion2dADMM(f, K, Kh, alpha, iterate_param, visual_param)
% 
%Input:
% f:              data
% K:              forward operator (function handle or matrix)
% Kh:             adjoint operator (function handle or matrix)
% alpha:          penalization parameter
% iterate_param:  parameters concerning iteration (see function parseInput 
%                 for details)
% visual_param:   parameters concerning visulization (see function 
%                 parseInput for details)
%
%Output:
% u:  solution
%Note: Extend the codes from Markus Grasmair to accept arbitrary forward 
%      operators

% Housen Li @ MPIbpc
% 30.09.2015 create the code
% 05.01.2016 change name "tvInv_ADMM" --> "tvInversion2dADMM"

if ~isa(K, 'function_handle')
    K = @(par) K * par;
end
if ~isa(Kh, 'function_handle')
    Kh = @(par) Kh * par;
end
if nargin < 5
    iterate_param = [];
end
if nargin < 6
    visual_param = [];
end
[sigma, max_steps, eps_main, ...
    max_steps_denoising, eps_denoising, ...
    max_steps_CG, eps_CG, ...
    vis_level, verbose, check] = parseInput(iterate_param, visual_param);

% convert the input data to double format if (for some reason) it is not
f = double(f);
% % determine at which angles the Radon transform has been evaluated
% % NB: in the computation of the adjoints it is always assumed that the
% % Radon transform has been evaluated at equidistant angles starting at 0.
% % If this part of the code is changed, then all the adjoints have to be
% % corrected as well.
% s_f    = size(f);
% angles = (0:(s_f(2)-1))*180/s_f(2);
% rescale_factor_iradon = 2/pi; % 1;
% interp = 'linear';

%% initialisations and memory allocations
% convergence flag and step number for the main iteration
not_converged = 1;
iterate_main  = 0;
% store the (unfiltered) backprojection of the data for future use
% f_back = iradon(f,[],interp,'None',1,imsize)*rescale_factor_iradon;
f_back = Kh(f);
% initialise the solution u by (unfiltered) backprojection
u = f_back;
% initialise the auxiliary variable v as equal to u
v = u;
% initialisation of the Lagrange parameter
lambda = zeros(size(u));

%% main iteration
while (not_converged)
    % store the old values of u and v
    u_old = u;
    v_old = v;
    
    current_verbose = verbose && (mod(iterate_main, check) == 0);
    if (current_verbose) 
        fprintf('  iteration %d ...\n', iterate_main);
    end
    
    %%% solve the denoising problem (update of u)
    % algorithm from "Bect, J., Blanc-F?raud, L., Aubert, G., and
    %   Chambolle, A., A $l$1-unified variational framework for image 
    %   restoration. Computer Vision - ECCV 2004 Volume 3024 of the series 
    %   Lecture Notes in Computer Science pp 1-13"
    % initialisations - in the case of large regularisation parameter, it
    % might make sense to improve the initialisation, else the number of
    % iterations will be huge
    u = tvRegression2dPG(v+lambda, alpha/sigma, u, max_steps_denoising, eps_denoising, current_verbose);
    
    %%% invert the Radon transform (update of v)
    v = tikhonovInv(f, K, Kh, sigma, u-lambda, v, max_steps_CG, eps_CG, current_verbose);
    
    %%% update of lambda
    lambda = lambda + (v - u);
    
    iterate_main = iterate_main + 1;
    
    if (iterate_main >= max_steps)
        not_converged = 0;
    end
    increment_u    = max(max(abs(u_old-u)));
    increment_v    = max(max(abs(v_old-v)));
    difference_u_v = max(max(abs(u-v)));
    if (current_verbose)
        fprintf('      increment_u = %g, increment_v = %g, diff_u_v = %g\n',...
            increment_u, increment_v, difference_u_v);
    end
    
    if (increment_u <= eps_main && increment_v <= eps_main && difference_u_v <= eps_main)
        not_converged = 0;
    end
    
    if (vis_level > 1)
        figure(1);clf;
        subplot(131); imagesc(u);      axis off; colorbar; drawnow;
        subplot(132); imagesc(v);      axis off; colorbar; drawnow;
        subplot(133); imagesc(lambda); axis off; colorbar; drawnow;
    end
end

if (verbose)
    fprintf('number of main iterations: %d\n', iterate_main);
end

if (vis_level > 0)
    figure(1);clf;
    subplot(131); imagesc(u);      axis off; colorbar; drawnow;
    subplot(132); imagesc(v);      axis off; colorbar; drawnow;
    subplot(133); imagesc(lambda); axis off; colorbar; drawnow;
end

% Equivalent optimization problem
%
%   minimize_{u, v} 1/2 || K v - f ||_2^2 + alpha || \nabla u ||_1
%   subject to v - u = 0
%
% Algorithm: Alternating Direction Method of Multipliers (ADMM)
%   augmented Lagrangian
%       L_{sigma}(u, v, lambda) = 1/2||K v - f||_2^2 + alpha||\nabla u||_1
%                             + sigma<lambda, v - u> + sigma/2||v - u||_2^2
%   iterations
%       u      := argmin_{u} 1/2||u - v - lambda||_2^2 
%                                + alpha/sigma ||\nabla u||_1
%       v      := argmin_{v} 1/(2 sigma)||K v - f||_2^2 
%                                + 1/2||v - u + lambda||_2^2
%       lambda := lambda + (v - u)

%% parsing input
function [sigma, max_steps, eps_main, ...
    max_steps_denoising, eps_denoising, ...
    max_steps_CG, eps_CG, ...
    vis_level, verbose, check] = parseInput(iterate_param, visual_param)

% default iteration parameters
% main problem
sigma     = 5;         % parameter for the augmented Lagrangian
max_steps = 500;       % maximal number of steps
eps_main  = 2*10^(-3); % stopping criterion
% denoising problem
max_steps_denoising = 500;     % maximal number of steps
eps_denoising       = 10^(-3); % stopping criterion
% tau                 = 0.24;    % step size for the projected gradient method
% Radon inversion 
max_steps_CG = 5e3; % 200; % maximal number of steps for the CG method
eps_CG       = 10^(-4);    % stopping criterion for the CG method

if (nargin > 0 && ~isempty(iterate_param)) 
    if (isfield(iterate_param, 'sigma'))
        sigma = iterate_param.sigma;
    end
    if (isfield(iterate_param, 'max_steps'))
        max_steps = iterate_param.max_steps;
    end
    if (isfield(iterate_param, 'eps_main'))
        eps_main = iterate_param.eps_main;
    end
    if (isfield(iterate_param, 'max_steps_denoising'))
        max_steps_denoising = iterate_param.max_steps_denoising;
    end
    if (isfield(iterate_param, 'eps_denoising'))
        eps_denoising = iterate_param.eps_denoising;
    end
    if (isfield(iterate_param, 'max_steps_CG'))
        max_steps_CG = iterate_param.max_steps_CG;
    end
    if (isfield(iterate_param, 'eps_CG'))
        eps_CG = iterate_param.eps_CG;
    end
end

% default visualization parameters
vis_level = 0;
verbose   = 0; % give some output during the computation
check     = 10;

if (nargin > 1 && ~isempty(visual_param))
    if (isfield(visual_param, 'vis_level'))
        vis_level = visual_param.vis_level;
    end
    if (isfield(visual_param, 'verbose'))
        verbose = visual_param.verbose;
    end
    if (isfield(visual_param, 'check'))
        check = visual_param.check;
    end
end
